%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  Exact.m
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 
%
% DESCRIPTION:
%   Exact line search
%
% INPUT:
%
% OUTPUT:
%
% USAGE:
%  [tr,mu_vec,cDDf, lower_step_bound,d_vec, stepsize_vec]=...
%       Exact(History, Hessian_product,mem,maxmem,iter,dim,...
%                x_mat, f_vec,gradf_mat,d_vec,mu_vec,stepsize_vec,...
%                tr,delta, alpha, eta1, eta2, eta3,cDDf,DDfnew,...
%                bDDf,orthog_TOL, step_TOL, lower_step_bound, gamma)
% 
% SUBROUTINE CALLS:
%      Hessian_product: invMSB2_product, invBFGS2_product, BFGS1_product,max
%                       and MSB1_product
%

function [tr,mu_vec,cDDf, lower_step_bound,d_vec, stepsize_vec]=Exact(History, Hessian_product,...
    mem,maxmem,iter,dim,...
    x_mat, f_vec,gradf_mat,d_vec,mu_vec,stepsize_vec,...
    tr,delta, alpha, eta1, eta2, eta3,cDDf,DDfnew, bDDf, orthog_TOL, lower_step_bound, gamma)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% exact line search via bisection
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(mu_vec(iter)==0)
    tr=1e+15;
    lower_step_bound=0;
    mu_vec(iter)=1;
end
min_d=lower_step_bound;
max_d=tr;
nd=stepsize_vec(iter);
% disp('in Exact'), keyboard

if((f_vec(iter+1) <= f_vec(iter))&&(DDfnew/nd<-orthog_TOL))
    % step too short:
    if abs(nd-max_d)/(max_d-min_d)<1e-2
        max_d=1e+7*max_d;
        tr=max_d;
        d_vec=2*d_vec;
    else
        % increase step length by one half the interval to the outer bound
        d_vec=(min((nd+max_d)/2,10*nd))*d_vec/nd;
    end
    min_d=nd;
elseif((f_vec(iter+1) <= f_vec(iter))&&(DDfnew/nd>orthog_TOL))
    % step too long, but not by too much:  decrease step length by one half the 
    %                                      interval to the inner bound
    d_vec=(nd+min_d)/2*(d_vec/nd);
    max_d=nd;
    tr=max_d;
elseif((f_vec(iter+1) > f_vec(iter)))
    % step too long, by a lot:  decrease step length by one half the current steplength 
    max_d=nd;
    tr=max_d;
    d_vec=0.5*(min_d+nd)*(d_vec/nd);
end

lower_step_bound = min_d;
stepsize_vec(iter)  = feval('norm',d_vec);
clear S_mat Y_mat x_mat gradf_mat delta alpha eta1 eta2 eta3 bDDf gamma iter maxmem
clear History Hessian_product
return
