function xnew_vec = SDBktk_rvrs(x_vec,gradf_vec,ngradf,gamma)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Backtrack_rvrs.m
%              written April 7, 1996  
%                   Russell Luke
%             Department of Mathematics
%              University of Washington  
%
%
%
%
% DESCRIPTION:
%
% This function performs the direction determination and backtracking subroutine using
% reverse communication.
% The backtracking routine attempts to find a step size for
% reducing the value of the function fname given the current point xold_vec
% and a direction d_vec. It does this by successively trying step sizes
% of the form gamma^s for s = 0,1,2... to find the smallest
% value of s for which the inequality
%
%       fname(xold_vec+gamma^s*d_vec)\le fname(xold_vec)+c*gamma^s*DDf
%
% is satisfied. The code is implemented according to the principles
% of reverse communication, which means that this backtracking
% routine is `dumb' and knowns nothing more about the function than
% is passed to it by the controlling routine.  This is not the most
% straight-forward coding style, but is much more general. 
% The new point to be returned is then given
% by xnew_vec = xold_vec+gamma^s*d_vec.
% The backtracking is terminated if the step to the new point
% xnew_vec is so small that it triggers termination in the main algorithm,
% i.e. norm(xold_vec - xnew_vec) <= eps. In this case we return xnew_vec = xold_vec if
% fnew >= fold (we have not reduced the function value); otherwise,
% we return xnew_vec.
%
% INPUT:
%
%       x_vec   = current point,
%       gradf_vec   = gradient of the objective,
%       ngradf = norm of gradf_vec
%       gamma = the backstepping parameter in (0,1),
%
% OUTPUT:
%       xnew_vec  = new point,
%       
%
% SUROUTINE CALLS:
%       none
%
% USAGE:
%
%      xnew_vec = SDBktk_rvrs(x_vec,gradf_vec,ngradf,gamma)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Determine the search direction: steepest descent
%
xnew_vec = x_vec - gamma*gradf_vec/ngradf;

