%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  Update_criteria.m
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 

% DESCRIPTION:  
%
% INPUT: 
%
% OUTPUT:
%   accept has 4 possible values: -2,-1, 0 and 1.  The -2 flag is used for 
% exact line search where many iterations might be needed to get the 
% step-length correct, but we don't want to update the memory with the 
% new function and gradient information collected as a result.  
% For backtracking line search and trust regions, it is reasonable
% to keep information collected by trial and error, and so ``rejected"
% steps are still used to update the memory -- accept_step=0 -- 
% and therefore change the step direction in the NEXT iteartion of the 
% corresponding subproblems).  
% The -1 flag is used for trust region algorithms where we do not update
% the memory, but we DO change the trust region radius, so a recalculation
% of the step is needed.
%
% USAGE: 
%      
% 
% SUBROUTINE CALLS:
%

function accept = Update_criteria(update_conditions,mem, iter, lookback,...
                                  f_vec,cDDf,DDfnew,bDDf,stepsize_vec,orthog_TOL,...
                                  mu_vec, eta4)

if(strcmp(update_conditions,'Exact'))
    if((abs(DDfnew)/stepsize_vec(iter)<=orthog_TOL))
        accept=1;
    else
        accept=-2;
    end
elseif(strcmp(update_conditions,'Trust Region'))
    if(mem>0)
        p=max(lookback,0); % lookback for nonmonotone decrease
        if (((mu_vec(iter)>eta4)||((f_vec(iter+1) <= max(f_vec(max(1,iter-p):iter))+cDDf)&&(DDfnew>=bDDf))))
            accept=1;
        elseif(f_vec(iter+1) <= max(f_vec(max(1,iter-p):iter)))&& (stepsize_vec(iter)<2*stepsize_vec(iter-1))
                accept=0; % still got decrease, so will keep the information, but not advance the point
        else
            accept=-1; % no decrease and stepsize might be too small, shrink memory and don't keep info
        end
    else % memory gone...doing linesearch
        if ((f_vec(iter+1) <= max(f_vec(max(1,iter-1):iter))+cDDf)&&(DDfnew>=bDDf))
            accept=1;
        else
            accept=-2;
        end
    end
end
clear f_vec cDDf DDfnew bDDf stepsize_vec p mem iter update_conditions
return