%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                  Burke_dvr.m
%                   Russell Luke
%  Department of MathSci, University of Delaware  
%                        and 
%  Inst. f. Num. u. Angew. Math., Universitaet Goettingen
%                    JUNE. 2, 2010  
%
%  This work was supported by NSF-DMS grant 0712796. 
%
%
% DESCRIPTION:  driver template for the SAMSARA reverse communication 
%               optimization toolbox.
%
% INPUT:  none...this is a script
%
% OUTPUT:
%       xnew_vec,  optimal solution
%       fnew, optimal value
%       diagnost_struct, data structure of diagnostic details, e.g. f_vec,
%                     ngrad_vec, stepsize_vec, iter and x_mat.
%
% USAGE: 
%        sample_dvr
% 
% SUBROUTINE CALLS:
%      SAMSARA
%      Objective
%

clear;
% add samsara toolbox to directory path
addpath('/home/r.luke/r.luke/work/matlab/tools/SAMSARA');
% addpath('/Users/rluke/work/matlab/tools/SAMSARA');
% clear out old data files
!rm *.dat

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function to minimize
%%%%%%%%%%%%%%%%%%%%%%%%%%%
Objective   = 'Burke_func';  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x0_vec=zeros(7,1);
% [0.09;0.09*ones(499,1);0.09;0.09*ones(499,1)];
dim           = length(x0_vec);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Tolerances:  note that there is a 
%     samsara toerlances OPTIONS_tolerances
%     file that might not match the tolerances
%     you specify here.  Copy OPTIONS_tolernaces
%     into the local directory where the ``driver"
%     program is and set the tolerances with this
%     input file.  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ngrad_TOL = 1e-12;
step_TOL = 1e-14;
Maxit=300;

%%%%%%%%%%%%%%%%%
% Main loop
%%%%%%%%%%%%%%%%%
iter=1;
xold_vec = x0_vec;
% SAMSARA needs two points:  the current point and a proposed point.  
% On the very first iteration the user can either generate the second 
% proposed point manually, or allow SAMSARA to automatically generate 
% the second point.  SAMSARA generates the second point by taking a scaling 
% of the steepest descent direction.  After the first iteration the current step is
% stored and updated in a file maintained by SAMSARA.  
[fold,gradfold_vec]=feval(Objective,xold_vec);
% fold=[];
% d_vec=-.1*gradfold_vec;
% xnew_vec=xold_vec+d_vec;
% stepsize=feval('norm',d_vec)
% [fnew,gradfnew_vec]=feval(Objective,xnew_vec);
% ngradfnew = sqrt(gradfnew_vec'*gradfnew_vec);
d_vec=[];
xnew_vec=[];
stepsize=999;
fnew=[];
gradfnew_vec = [];
ngradfnew=999;
while ((iter<Maxit)&&(ngradfnew>ngrad_TOL)&&(stepsize>step_TOL))
    [xnew_vec,xold_vec,fold,gradfold_vec, stepsize]=...
        feval('samsara', xold_vec, xnew_vec, fold, fnew, gradfold_vec, gradfnew_vec);
    iter=iter+1;
    % new function evaluation:
    [fnew,gradfnew_vec] = feval(Objective,xnew_vec);
    ngradfnew = sqrt(gradfnew_vec'*gradfnew_vec);
end
if(stepsize<=step_TOL)
    disp('Algorithm stagnated:  stepsize tolerance violated.')
end
%%%%%%%%%%%%%%%%%
% end main loop
%%%%%%%%%%%%%%%%%
xopt_vec=xnew_vec;
fopt=fnew;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% prepare diagnostic data and return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[fid,message]=fopen('state.dat','r');
if (fid<0)
    disp(message)
else
    state_vec=fread(fid,[1,inf],'real*8');
    fclose(fid);
end
Maxit=state_vec(end);
f_vec = state_vec(1:Maxit+1);
ngradf_vec = state_vec(Maxit+2:2*(Maxit+1));
stepsize_vec = state_vec(2*Maxit+3:3*(Maxit+1));
mu_vec = state_vec(3*Maxit+4:4*(Maxit+1));
iter = state_vec(4*(Maxit+1)+1);
fcounter = state_vec(4*(Maxit+1)+10);
trstep = state_vec(4*(Maxit+1)+11);
samsara_diagnostic_viewer


display(['iterations: ',num2str(iter),'; function evaluations: ',num2str(fcounter),';  optimal value: ' num2str(fopt)])



